<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"><head>

<meta content="text/html; charset=UTF-8" http-equiv="content-type" />
<title>FusionInventory-Agent</title>
<link rel="stylesheet" href="site.css" type="text/css" />

</head>
<body>
<p>{$message}</p>
<p><a href="/">Back</a></p>
</body>
</html>
